/**
 *
 * @file bl_communication_interface.c
 *
 * @ingroup generic_bootloader_8bit
 *
 * @brief This source file provides the implementation of the communication interface API prototypes required by the 8-bit Bootloader library
 *
 * @version BOOTLOADER Driver Version 3.0.0
 */

${disclaimer}

#include <stdint.h>
#include <stdbool.h>
#include "../bl_communication_interface.h"

#define  STX   0x55

#define USART_IsRxReady()                       ${BL_UARTIsRxReady}()
#define USART_Read()                            ${BL_UARTRead}()
#define USART_Write(data)                       ${BL_UARTWrite}(data)
#define USART_IsTxReady()                       ${BL_UARTIsTxReady}()
#define USART_IsTxDone()                        ${BL_UARTTransmitDone}()
#define UART_AutoBaudSet(enable)                ${BL_UARTAutoBaudSet}(enable)
#define UART_AutoBaudQuery()                    ${BL_UARTAutoBaudQuery}()
#define USART_AutoBaudDetectErrorReset()        ${BL_USARTAutoBaudDetectErrorReset}()
#define USART_IsAutoBaudDetectError()           ${BL_USARTIsAutoBaudDetectError}()


void BL_CommunicationModuleInit(void)
{
    <#if DEVICE_TYPE == "PIC">
    <#if ( (DRAIN_RXBUF_AFTER_AB == "Enabled") && (CHIP_TYPE == "PIC18") ) || (CHIP_TYPE == "PIC16")>
    uint16_t  clearCh;
    </#if>  
    </#if>

    UART_AutoBaudSet(true);

    while(UART_AutoBaudQuery() != 1)
    {
        if( USART_IsAutoBaudDetectError() == true)
        {
            UART_AutoBaudSet(false);
            USART_AutoBaudDetectErrorReset();
            UART_AutoBaudSet(true); 
        }
    }
<#if DEVICE_TYPE == "PIC">
<#if ( (DRAIN_RXBUF_AFTER_AB == "Enabled") && (CHIP_TYPE == "PIC18") ) || (CHIP_TYPE == "PIC16")>
    clearCh = USART_Read();  // required to clear RCIF
</#if>  
</#if>
}

bool BL_CommunicationModuleIsReady(void)
{
    bool status;

    if (USART_IsTxDone())
    {
        status = true;
    }
    else
    {
        status = false;
    }

    return status;
}

void BL_CommunicationModuleRead(uint8_t *data, size_t dataLength)
{
    size_t commReadDataCount;
    commReadDataCount = 0;

    while (commReadDataCount < dataLength)
    {
        if (USART_IsRxReady())
        {
            *data++ = USART_Read();
            commReadDataCount++;
        }
    }
}

void BL_CommunicationModuleWrite(uint8_t *data, size_t dataLength)
{
    size_t commWriteDataCount;
    commWriteDataCount = 0;

    USART_Write(STX);
    while (commWriteDataCount < dataLength)
    {
        if (USART_IsTxReady())
        {
            USART_Write(*data);
            commWriteDataCount++;
            data++;
        }

    }
}
