import { getters } from "./BootLogic";
import { BootTemplatePayload, VERIFICATION_SCHEME_VALUE } from "./BootTemplateInterfaces";
import { decimal2Hex } from "./Helpers";
import { DeviceTestData, PinsTestData } from "./testData";

describe("Device Specific Helper Tests", () => {
    it("test function getResetBits", () => {
        expect(getters.getResetBits).toBeDefined();
        expect(
            getters.getResetBits(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
            ),
        ).toBe("RSTCTRL_SWRST_bm");
        expect(
            // AVR64 Tests for Reset Bit
            getters.getResetBits(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[1],
            ),
        ).toBe("RSTCTRL_SWRST_bm");
        expect(
            // ATtiny test
            getters.getResetBits(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[4],
            ),
        ).toBe("RSTCTRL_SWRE_bm");
        expect(
            getters.getResetBits(
                DeviceTestData.PIC16.architecure,
                DeviceTestData.PIC16.supportedDevices[0],
            ),
        ).toBe("NotApplicable");
        expect(
            getters.getResetBits(
                DeviceTestData.PIC18.architecure,
                DeviceTestData.PIC18.supportedDevices[0],
            ),
        ).toBe("NotApplicable");
    });

    it("test function getChipType", () => {
        expect(getters.getChipType).toBeDefined();

        expect(
            getters.getChipType(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
            ),
        ).toBe("AVR");
        expect(getters.getChipType("AVR8", "AVR128DA48")).toBe("AVR");
        expect(getters.getChipType("AVR8X", "ATtiny1607")).toBe("TINY");
        expect(getters.getChipType(DeviceTestData.PIC16.architecure, "PIC16F15276")).toBe(
            "PIC16",
        );
        expect(getters.getChipType(DeviceTestData.PIC18.architecure, "PIC18F57Q43")).toBe(
            "PIC18",
        );
        expect(getters.getChipType("PIC12", "no affect")).toBe("PIC12");
    });

    it("test function convertUIArgumentsToUserData", () => {
        expect(getters.getDeviceSpecificAlerts).toBeDefined();
        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.PIC18.architecure,
                DeviceTestData.PIC18.supportedDevices[0],
                {
                    bootFuse: 0x0,
                    codeFuse: 0x0,
                    bootAddressValid: true,
                },
            ),
        ).toEqual([]);
        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.PIC18.architecure,
                DeviceTestData.PIC18.supportedDevices[0],
                {
                    bootFuse: 0x0,
                    codeFuse: 0x0,
                    bootAddressValid: false,
                },
            ),
        ).toEqual([
            {
                type: "error",
                text: "Boot End Address cannot be set to 0x0",
            },
        ]);
        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
                {
                    bootFuse: 0x0,
                    codeFuse: 0x0,
                    bootAddressValid: false,
                },
            ),
        ).toEqual([
            {
                type: "warning",
                text: "Recommended BOOTSIZE Range: Minimum Size - Maximum Size\n\nOptimization 0 -> 0x0E - 0x13\nOptimization 2 -> 0x06 - 0x0A",
            },
        ]);
        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.AVR.architecure,
                "ATtiny1607",
                {
                    bootFuse: 0x0,
                    codeFuse: 0x0,
                    bootAddressValid: false,
                },
            ),
        ).toEqual([
            {
                type: "warning",
                text: "Recommended BOOTEND Range: Minimum Size - Maximum Size\n\nOptimization 0 -> 0x1C - 0x2D\nOptimization 2 -> 0x0B - 0x0D",
            },
        ]);

        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.AVR.architecure,
                "ATtiny1607",
                {
                    bootFuse: 0x0,
                    codeFuse: 0x10,
                    bootAddressValid: false,
                },
            ),
        ).toEqual([
            {
                type: "warning",
                text: "Recommended BOOTEND Range: Minimum Size - Maximum Size\n\nOptimization 0 -> 0x1C - 0x2D\nOptimization 2 -> 0x0B - 0x0D",
            },
            {
                type: "error",
                text: "The CODEEND (16) should be a larger value than the BOOTEND (0). The current FUSE configuration will cause issues while programming the Bootloader",
            },
        ]);

        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.AVR.architecure,
                "ATtiny1607",
                {
                    bootFuse: 0x0c,
                    codeFuse: 0x00,
                    bootAddressValid: false,
                },
            ),
        ).toEqual([
            {
                type: "hint",
                text: "Recommended BOOTEND Range: Minimum Size - Maximum Size\n\nOptimization 0 -> 0x1C - 0x2D\nOptimization 2 -> 0x0B - 0x0D",
            },
        ]);

        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
                {
                    bootFuse: 0x10,
                    codeFuse: 0x08,
                    bootAddressValid: false,
                },
            ),
        ).toEqual([
            {
                type: "hint",
                text: "Recommended BOOTSIZE Range: Minimum Size - Maximum Size\n\nOptimization 0 -> 0x0E - 0x13\nOptimization 2 -> 0x06 - 0x0A",
            },
            {
                type: "error",
                text: "The CODESIZE (8) should be a larger value than the BOOTSIZE (16). The current FUSE configuration will cause issues while programming the Bootloader",
            },
        ]);
        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
                {
                    bootFuse: 0x0,
                    codeFuse: 0x08,
                    bootAddressValid: false,
                },
            ),
        ).toEqual([
            {
                type: "warning",
                text: "Recommended BOOTSIZE Range: Minimum Size - Maximum Size\n\nOptimization 0 -> 0x0E - 0x13\nOptimization 2 -> 0x06 - 0x0A",
            },
            {
                type: "error",
                text: "The CODESIZE (8) should be a larger value than the BOOTSIZE (0). The current FUSE configuration will cause issues while programming the Bootloader",
            },
        ]);
        expect(
            getters.getDeviceSpecificAlerts(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
                {
                    bootFuse: 0x12,
                    codeFuse: 0x0,
                    bootAddressValid: true,
                },
            ),
        ).toEqual([
            {
                type: "hint",
                text: "Recommended BOOTSIZE Range: Minimum Size - Maximum Size\n\nOptimization 0 -> 0x0E - 0x13\nOptimization 2 -> 0x06 - 0x0A",
            },
        ]);
    });

    it("test function getDeviceSpecificConfiguration", () => {
        expect(getters.getConfiguration).toBeDefined();
        expect(getters.getConfiguration(DeviceTestData.AVR.architecure, "0")).toEqual([
            {
                group: "HI-TECH-COMP",
                key: "extra-include-directories",
                value: "mcc_generated_files/bootloader;mcc_generated_files",
            },
            {
                group: "AVR-GCC",
                key: "optimization-level",
                value: "-O2",
            },
            {
                group: "HI-TECH-COMP",
                key: "optimization-level",
                value: "-O2",
            },
        ]);
        expect(
            getters.getConfiguration(DeviceTestData.PIC16.architecure, "0-3FF"),
        ).toEqual([
            {
                group: "HI-TECH-COMP",
                key: "extra-include-directories",
                value: "mcc_generated_files/bootloader;mcc_generated_files",
            },
            {
                group: "HI-TECH-COMP",
                key: "optimization-level",
                value: "-O2",
            },
            {
                group: "HI-TECH-LINK",
                key: "code-model-rom",
                value: "0-3FF",
            },
        ]);
        expect(
            getters.getConfiguration(DeviceTestData.PIC18.architecure, "0-3FFF"),
        ).toEqual([
            {
                group: "HI-TECH-COMP",
                key: "extra-include-directories",
                value: "mcc_generated_files/bootloader;mcc_generated_files",
            },
            {
                group: "HI-TECH-COMP",
                key: "optimization-level",
                value: "-O2",
            },
            {
                group: "HI-TECH-LINK",
                key: "code-model-rom",
                value: "0-3FFF",
            },
        ]);
        expect(
            getters.getConfiguration(DeviceTestData.PIC18.architecure, undefined),
        ).toEqual([
            {
                group: "HI-TECH-COMP",
                key: "extra-include-directories",
                value: "mcc_generated_files/bootloader;mcc_generated_files",
            },
            {
                group: "HI-TECH-COMP",
                key: "optimization-level",
                value: "-O2",
            },
        ]);
        expect(getters.getConfiguration(DeviceTestData.PIC16.architecure, "0")).toEqual([
            {
                group: "HI-TECH-COMP",
                key: "extra-include-directories",
                value: "mcc_generated_files/bootloader;mcc_generated_files",
            },
            {
                group: "HI-TECH-COMP",
                key: "optimization-level",
                value: "-O2",
            },
        ]);
        expect(getters.getConfiguration(DeviceTestData.AVR.architecure, "0")).toEqual([
            {
                group: "HI-TECH-COMP",
                key: "extra-include-directories",
                value: "mcc_generated_files/bootloader;mcc_generated_files",
            },
            {
                group: "AVR-GCC",
                key: "optimization-level",
                value: "-O2",
            },
            {
                group: "HI-TECH-COMP",
                key: "optimization-level",
                value: "-O2",
            },
        ]);
    });

    it("test function getFuse", () => {
        expect(getters.getFuse).toBeDefined();
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "AVR128DA48")).toBe(
            "Reserved_Two_Byte",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "AVR64DA48")).toBe(
            "Reserved_Two_Byte",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "ATmega1264")).toBe(
            "NotApplicable",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "attiny1607")).toBe(
            "Reserved_Two_Byte",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "attiny3217")).toBe(
            "Reserved_One_Byte",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "att3217")).toBe(
            "NotApplicable",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "attiny1627")).toBe(
            "NotApplicable",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "att1627")).toBe(
            "NotApplicable",
        );
        expect(getters.getFuse(DeviceTestData.AVR.architecure, "UNKNOWN")).toBe(
            "NotApplicable",
        );
        expect(getters.getFuse(DeviceTestData.PIC16.architecure, "SHOULDSHORTCUT")).toBe(
            "NotApplicable",
        );
    });

    it("test function getNVMAccessType", () => {
        expect(getters.getNVMAccessType).toBeDefined();
        expect(getters.getNVMAccessType("ATtiny1607")).toBe("ROW_ACCESS");
        expect(getters.getNVMAccessType("AVR128DA48")).toBe("BYTE_ACCESS");
        expect(getters.getNVMAccessType("unknownDeviceName")).toBe("BYTE_ACCESS"); // default return
    });

    it("test function getAVRPage", () => {
        expect(getters.getAVRPage).toBeDefined();

        expect(
            getters.getAVRPage(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
            ),
        ).toBe("Expanded");

        expect(getters.getAVRPage(DeviceTestData.AVR.architecure, "AVR64DA64")).toBe(
            "Expanded",
        );

        expect(getters.getAVRPage(DeviceTestData.AVR.architecure, "AVR32DA28")).toBe(
            "Expanded",
        );

        expect(getters.getAVRPage(DeviceTestData.AVR.architecure, "attiny1627")).toBe(
            "Classic",
        );

        expect(
            getters.getAVRPage(DeviceTestData.PIC16.architecure, "DOESNOTMATTER"),
        ).toBe("NotApplicable");

        expect(
            getters.getAVRPage(DeviceTestData.PIC18.architecure, "DOESNOTMATTER"),
        ).toBe("NotApplicable");
    });

    it("test function getEEPROMSize", () => {
        expect(getters.getEEPROMSize).toBeDefined();

        expect(getters.getEEPROMSize(256)).toBe("Classic");
        expect(getters.getEEPROMSize(512)).toBe("Extended");
        expect(getters.getEEPROMSize(0)).toBe("Classic");
    });

    it("test function getBootBlockSize", () => {
        expect(getters.getBootBlockSize).toBeDefined();
        expect(getters.getBootBlockSize("attiny1607")).toBe("256bytes");
        expect(getters.getBootBlockSize("ATtiny3217")).toBe("256bytes");
        expect(getters.getBootBlockSize("AVR128DA48")).toBe("512bytes");
        expect(getters.getBootBlockSize("PIC18F57Q43")).toBe("512bytes");
    });

    it("test function getRegisterNames", () => {
        expect(getters.getRegisterNames).toBeDefined();

        expect(
            getters.getRegisterNames(DeviceTestData.PIC16.architecure).EECON2_SFR,
        ).toBe("NVMCON2");
        expect(
            getters.getRegisterNames(DeviceTestData.PIC18.architecure).EECON2_SFR,
        ).toBe("NVMLOCK");
    });

    it("test function getNoAppCode", () => {
        expect(getters.getNoAppCode).toBeDefined();
        expect(getters.getNoAppCode(DeviceTestData.PIC16.architecure)).toBe("0x3FFF");
        expect(getters.getNoAppCode(DeviceTestData.PIC18.architecure)).toBe("0xFF");
        expect(getters.getNoAppCode(DeviceTestData.AVR.architecure)).toBe("0xFF");
    });

    it("test function getSupportConfigWrite", () => {
        expect(getters.getSupportConfigWrite).toBeDefined();

        expect(
            getters.getSupportConfigWrite(
                DeviceTestData.PIC16.architecure,
                DeviceTestData.PIC16.supportedDevices[0],
            ),
        ).toBe("Enabled");

        expect(
            getters.getSupportConfigWrite(
                DeviceTestData.PIC16.architecure,
                "PIC16LF18313",
            ),
        ).toBe("Enabled");

        expect(
            getters.getSupportConfigWrite(
                DeviceTestData.PIC18.architecure,
                DeviceTestData.PIC18.supportedDevices[0],
            ),
        ).toBe("Enabled");

        expect(
            getters.getSupportConfigWrite(
                DeviceTestData.AVR.architecure,
                DeviceTestData.AVR.supportedDevices[0],
            ),
        ).toBe("Disabled");
    });

    it("test function getDeviceFamily", () => {
        expect(getters.isPICKXXDeviceFamily).toBeDefined();

        expect(getters.isPICKXXDeviceFamily("PIC18F26Q10")).toBe("NotApplicable");
        expect(getters.isPICKXXDeviceFamily("PIC18F46K42")).toBe("K42");
        expect(getters.isPICKXXDeviceFamily("PIC18F26K83")).toBe("K83");
    });

    it("test function getDrainRXBUF", () => {
        expect(getters.getDrainRXBUF).toBeDefined();
        expect(
            getters.getDrainRXBUF(
                DeviceTestData.PIC18.architecure,
                DeviceTestData.PIC18.supportedDevices[0],
            ),
        ).toBe("Disabled");
        expect(
            getters.getDrainRXBUF(DeviceTestData.PIC18.architecure, "PIC18F25K22"),
        ).toBe("Enabled");
        expect(
            getters.getDrainRXBUF(DeviceTestData.PIC18.architecure, "PIC18F25K83"),
        ).toBe("Enabled");
        expect(
            getters.getDrainRXBUF(DeviceTestData.PIC18.architecure, "PIC18F25K42"),
        ).toBe("Disabled");
        expect(
            getters.getDrainRXBUF(DeviceTestData.PIC18.architecure, "PIC18F25K40"),
        ).toBe("Enabled");
        expect(
            getters.getDrainRXBUF(
                DeviceTestData.PIC16.architecure,
                DeviceTestData.PIC16.supportedDevices[0],
            ),
        ).toBe("NotApplicable");
    });

    it("test function getDeviceType", () => {
        expect(getters.getDeviceType).toBeDefined();
        expect(getters.getDeviceType(DeviceTestData.AVR.architecure)).toBe("AVR");
        expect(getters.getDeviceType(undefined)).toBe("NotApplicable");
        expect(getters.getDeviceType(DeviceTestData.PIC16.architecure)).toBe("PIC");
        expect(getters.getDeviceType(DeviceTestData.PIC18.architecure)).toBe("PIC");
    });

    it("test function getDeviceIdStart", () => {
        expect(getters.getDeviceIdStart).toBeDefined();
        expect(getters.getDeviceIdStart(DeviceTestData.AVR.architecure)).toBe("0x3FFFFE");
        expect(getters.getDeviceIdStart(DeviceTestData.PIC16.architecure)).toBe("0x00");
    });

    it("test function getInterruptVectorRange", () => {
        expect(getters.getInterruptVectorRange).toBeDefined();
        expect(
            getters.getInterruptVectorRange(
                800,
                "high",
                DeviceTestData.PIC16.architecure,
            ),
        ).toBe(decimal2Hex(800 + 4));
        expect(
            getters.getInterruptVectorRange(
                800,
                "high",
                DeviceTestData.PIC18.architecure,
            ),
        ).toBe(decimal2Hex(800 + 8));
        expect(
            getters.getInterruptVectorRange(800, "low", DeviceTestData.PIC16.architecure),
        ).toBe(decimal2Hex(800 + 4));
        expect(
            getters.getInterruptVectorRange(800, "low", DeviceTestData.PIC18.architecure),
        ).toBe(decimal2Hex(800 + 24));
        expect(
            getters.getInterruptVectorRange(800, "low", DeviceTestData.PIC16.architecure),
        ).toBe("0x324");
        expect(
            getters.getInterruptVectorRange(800, "low", DeviceTestData.PIC18.architecure),
        ).toBe("0x338");
    });
});

describe("User Interface Helper Tests", () => {
    it("Test getCRCData function.", () => {
        const testCRC16Data: BootTemplatePayload = {
            CRC_SEED_U: "0xFFFF",
            CRC_SIZE_U: "2",
            CRC_POLYNOMIAL_U: "0x1021",
            CRC_XOR_U: "0x00000000",
        };
        const testCRC32Data: BootTemplatePayload = {
            CRC_SEED_U: "0xFFFFFFFF",
            CRC_SIZE_U: "4",
            CRC_POLYNOMIAL_U: "0xEDB88320",
            CRC_XOR_U: "0x00000000",
        };
        expect(getters.getCRCData).toBeDefined();
        expect(getters.getCRCData("CRC32_VERIFICATION")).toEqual(testCRC32Data);
        expect(getters.getCRCData("CRC16_VERIFICATION")).toEqual(testCRC16Data);
    });

    describe("Test template data from different verification schemes", () => {
        it("Should return the help links and ui dscriptions for the verification selector", () => {
            expect(getters.getVerification("Check State Flag")).toEqual(
                "STATUS_BYTE_VERIFICATION" as VERIFICATION_SCHEME_VALUE,
            );
            // Test CRC32
            expect(getters.getVerification("CRC-32")).toEqual(
                "CRC32_VERIFICATION" as VERIFICATION_SCHEME_VALUE,
            );

            // Test CRC16
            expect(getters.getVerification("CRC-16")).toEqual(
                "CRC16_VERIFICATION" as VERIFICATION_SCHEME_VALUE,
            );

            // Test Checksum
            expect(getters.getVerification("Checksum")).toEqual(
                "CHECKSUM_VERIFICATION" as VERIFICATION_SCHEME_VALUE,
            );

            // Test Failures
            expect(getters.getVerification("undefined")).toEqual("NotApplicable");
        });
    });
    it("Test getIndicatorActivation function.", () => {
        expect(getters.getIndicatorActivation).toBeDefined();
        expect(getters.getIndicatorActivation(PinsTestData.Good)).toBe("Active_Low");
        expect(getters.getIndicatorActivation(undefined)).toBe("Active_High");
        expect(getters.getIndicatorActivation(PinsTestData.Bad)).toBe("Active_High");
        expect(getters.getIndicatorActivation(PinsTestData.Empty)).toBe("Active_High");
        expect(getters.getIndicatorActivation("{}")).toBe("Active_High");
    });

    it("Test getCustomCname function.", () => {
        expect(getters.getCustomCname).toBeDefined();
        expect(getters.getCustomCname("ioOutPin", PinsTestData.Good)).toBe("TESTNAME");
        expect(getters.getCustomCname("ioOutPin", PinsTestData.Bad)).toBe("BL_INDICATOR");
        expect(getters.getCustomCname("ioOutPin", undefined)).toBe("BL_INDICATOR");
        expect(getters.getCustomCname("ioOutPin", "undefined")).toBe("BL_INDICATOR");
        expect(getters.getCustomCname("ioOutPin", PinsTestData.Empty)).toBe(
            "BL_INDICATOR",
        );
    });
});
