import * as BootTemplateInjector from "./BootTemplateInjector";
import { BootTemplatePayload, BootUserData } from "./BootTemplateInterfaces";
import {
    ATTINY1607_TestTemplateData,
    ATTINY3217_TestTemplateData,
    AVR128DA48_TestTemplateData,
    defaultTemplateData,
    MockDelayHeader,
    PIC18F57Q43_TestTemplateData,
    PinsTestData,
    testUARTPayload,
    testUSARTPayload,
} from "./testData";

describe("Template Injector Tests", () => {
    it("Should be valid on import", () => {
        expect(BootTemplateInjector).toBeDefined();
    });

    const emptyBootInjector = new BootTemplateInjector.BootTemplateInjector(
        {
            UART: undefined,
            I2C: undefined,
        },
        undefined,
        "undefined",
        undefined,
        {
            eeprom_size: 0,
            eeprom_start: 0,
            flash_size: 0,
        },
        {
            BootEntryIO: false,
            BootRunningIO: false,
            EERead: false,
            EEWrite: false,
            ConfigWrite: false,
            ConfigRead: false,
            EraseMode: "FullErase",
            verification: "Check Reset Vector",
            Checksum: false,
            EntryData: {},
            IndicatorData: {},
            FlashRead: false,
            SelfProtect: false,
            ResetVectorAddress: 0,
        } as BootUserData,
    );

    const AVR128DA48_BootInjector = new BootTemplateInjector.BootTemplateInjector(
        {
            UART: testUSARTPayload,
            I2C: undefined,
        },
        MockDelayHeader,
        "AVR128DA48",
        "AVR",
        {
            eeprom_start: 5120,
            eeprom_size: 512,
            flash_size: 131072,
        },
        {
            BootEntryIO: false,
            BootRunningIO: true,
            EERead: false,
            EEWrite: false,
            ConfigWrite: false,
            ConfigRead: false,
            EraseMode: "FullErase",
            verification: "Check Reset Vector",
            Checksum: false,
            EntryData: {},
            IndicatorData: PinsTestData.Good,
            FlashRead: false,
            SelfProtect: false,
            ResetVectorAddress: 0,
        } as BootUserData,
    );

    const ATTINY1607_BootInjector = new BootTemplateInjector.BootTemplateInjector(
        {
            UART: testUSARTPayload,
            I2C: undefined,
        },
        MockDelayHeader,
        "ATtiny1607",
        "AVR",
        {
            eeprom_start: 5120,
            eeprom_size: 256,
            flash_size: 49151,
        },
        {
            BootEntryIO: true,
            BootRunningIO: false,
            EERead: false,
            EEWrite: false,
            ConfigWrite: false,
            ConfigRead: false,
            EraseMode: "FullErase",
            verification: "Check Reset Vector",
            Checksum: false,
            EntryData: PinsTestData.Good,
            IndicatorData: {},
            FlashRead: false,
            SelfProtect: false,
            ResetVectorAddress: 0,
        } as BootUserData,
    );

    const ATTINY3217_BootInjector = new BootTemplateInjector.BootTemplateInjector(
        {
            UART: testUSARTPayload,
            I2C: undefined,
        },
        MockDelayHeader,
        "ATtiny3217",
        "AVR",
        {
            eeprom_start: 5120,
            eeprom_size: 256,
            flash_size: 65535,
        },
        {
            BootEntryIO: true,
            BootRunningIO: false,
            EERead: false,
            EEWrite: false,
            ConfigWrite: false,
            ConfigRead: false,
            EraseMode: "FullErase",
            verification: "Check Reset Vector",
            Checksum: false,
            EntryData: PinsTestData.Good,
            IndicatorData: {},
            FlashRead: false,
            SelfProtect: false,
            ResetVectorAddress: 0,
        } as BootUserData,
    );

    const PIC18F57Q43_BootInjector = new BootTemplateInjector.BootTemplateInjector(
        {
            UART: testUARTPayload,
            I2C: undefined,
        },
        MockDelayHeader,
        "PIC18F57Q43",
        "PIC18",
        {
            eeprom_start: 3670016,
            eeprom_size: 1024,
            flash_size: 131072,
        },
        {
            BootEntryIO: false,
            BootRunningIO: true,
            EERead: false,
            EEWrite: false,
            ConfigWrite: false,
            ConfigRead: true,
            EraseMode: "FullErase",
            verification: "Check Reset Vector",
            Checksum: false,
            EntryData: {},
            IndicatorData: PinsTestData.Good,
            FlashRead: false,
            SelfProtect: false,
            ResetVectorAddress: 5000,
        } as BootUserData,
    );

    it("Test that the constructor is working to create the default payload", () => {
        expect(emptyBootInjector).toBeDefined();
        expect(emptyBootInjector.inject).toBeDefined();
        expect(emptyBootInjector.setTemplateData).toBeDefined();
        expect(emptyBootInjector.inject()).toEqual(defaultTemplateData);
    });

    it("Test that the setTemplate api works", () => {
        AVR128DA48_BootInjector.setTemplateData();
        const data: BootTemplatePayload = AVR128DA48_BootInjector.inject();
        expect(data).toEqual(AVR128DA48_TestTemplateData);
    });

    it("Test that the setTemplate api works", () => {
        PIC18F57Q43_BootInjector.setTemplateData();
        const data: BootTemplatePayload = PIC18F57Q43_BootInjector.inject();
        expect(data).toEqual(PIC18F57Q43_TestTemplateData);
    });

    it("Test that the setTemplate api works", () => {
        ATTINY1607_BootInjector.setTemplateData();
        const data: BootTemplatePayload = ATTINY1607_BootInjector.inject();
        expect(data).toEqual(ATTINY1607_TestTemplateData);
    });

    it("Test that the setTemplate api works", () => {
        ATTINY3217_BootInjector.setTemplateData();
        const data: BootTemplatePayload = ATTINY3217_BootInjector.inject();
        expect(data).toEqual(ATTINY3217_TestTemplateData);
    });
});
