import { getModel, mockState } from "../generated_module/tests/AppModel.test";
import { mockAssignedModule } from "../generated_module/tests/processor.test";
import { getDerivedData } from "./DerivedData";
import * as PinAPI from "./PinsLogic";
import { testEntryData, testIndicatorData } from "./testData";

const initModel = (state: any, getterFunction: any): any => {
    const newModel = getModel(mockAssignedModule(state), getterFunction);
    return {
        model: newModel,
        derivedData: newModel?.getDerivedData(),
        derivedFunctions: newModel?.getDerivedData()?.getMyFunctions(),
    };
};

const defaultTestAPI = initModel(mockState(), getDerivedData);

describe("Pin logic tests", () => {
    it("Should load the APIs", () => {
        expect(PinAPI.getRowData).toBeDefined();
        expect(PinAPI.getCompletePinData).toBeDefined();
    });

    it("Should be able to return new actions", () => {
        expect(PinAPI.getRowData(defaultTestAPI.model, testIndicatorData)).toEqual({
            ...testIndicatorData,
            actions: [
                {
                    actiontype: "ATTRIBACTION",
                    state: {
                        L: {
                            payload: {
                                path: "hide.ioOutPin",
                                tabId: "main",
                                value: {
                                    attribs: ["cname", "high"],
                                },
                            },
                            type: "[FORM] SUBMIT",
                        },
                    },
                    validFor: ".*",
                },
                {
                    actiontype: "ATTRIBACTION",
                    state: {
                        UL: {
                            payload: {
                                path: "hide.ioOutPin",
                                tabId: "main",
                                value: {
                                    attribs: ["cname", "high"],
                                },
                            },
                            type: "[FORM] SUBMIT",
                        },
                    },
                    validFor: ".*",
                },
            ],
        });
        expect(PinAPI.getRowData(defaultTestAPI.model, testEntryData)).toEqual({
            ...testEntryData,
            actions: [
                {
                    actiontype: "ATTRIBACTION",
                    state: {
                        L: {
                            payload: {
                                path: "hide.ioInPin",
                                tabId: "main",
                                value: {
                                    attribs: ["cname"],
                                },
                            },
                            type: "[FORM] SUBMIT",
                        },
                    },
                    validFor: ".*",
                },
                {
                    actiontype: "ATTRIBACTION",
                    state: {
                        UL: {
                            payload: {
                                path: "hide.ioInPin",
                                tabId: "main",
                                value: {
                                    attribs: ["cname"],
                                },
                            },
                            type: "[FORM] SUBMIT",
                        },
                    },
                    validFor: ".*",
                },
            ],
        });

        expect(
            PinAPI.getRowData(defaultTestAPI.model, {
                ...testEntryData,
                name: "UNKNOWNPIN",
            }),
        ).toEqual({
            ...testEntryData,
            name: "UNKNOWNPIN",
            actions: [],
        });
    });
    it("Should load dynamic pins data from function", () => {
        expect(PinAPI.getCompletePinData(defaultTestAPI.model)).toEqual({ rows: [] });

        let localTestAPI = initModel(mockState(true), getDerivedData);

        expect(PinAPI.getCompletePinData(localTestAPI.model)).toEqual({
            rows: [testIndicatorData, testEntryData],
        });

        localTestAPI = initModel(
            {
                ...mockState(true),
                main: {
                    hide: {
                        ioOutPin: undefined,
                        ioInPin: undefined,
                    },
                },
            },
            getDerivedData,
        );

        expect(PinAPI.getCompletePinData(localTestAPI.model)).toEqual({
            rows: [],
        });
    });
});
