import { behaviour, direction, pin_row } from "@microchip/pin-standard";
import { HeaderFile } from "@microchip/scf-interface";
import { Architecture, BootTemplatePayload } from "./BootTemplateInterfaces";

export const getNotSupportedForArchTests = (archString: string): any => {
    const notSupported: string[] = Bootloader_Device_Lists.NOSUPPORT.Mega.concat(
        Bootloader_Device_Lists.NOSUPPORT.PIC18FQ10,
        Bootloader_Device_Lists.NOSUPPORT.PIC18FQ83,
        Bootloader_Device_Lists.NOSUPPORT.PIC18FQ84,
        Bootloader_Device_Lists.NOSUPPORT.PIC18FQ71,
        Bootloader_Device_Lists.NOSUPPORT.PIC18FQ20,
        Bootloader_Device_Lists.NOSUPPORT.PIC18FQ24,
        Bootloader_Device_Lists.NOSUPPORT.PIC18FK40,
        Bootloader_Device_Lists.NOSUPPORT.PIC18FK42,
        Bootloader_Device_Lists.NOSUPPORT.Tiny2,
        Bootloader_Device_Lists.NOSUPPORT.AVREA,
        Bootloader_Device_Lists.NOSUPPORT.AVRDB,
        Bootloader_Device_Lists.NOSUPPORT.AVRDU,
        Bootloader_Device_Lists.NOSUPPORT.AVRDD,
        Bootloader_Device_Lists.NOSUPPORT.PIC16F171,
        Bootloader_Device_Lists.NOSUPPORT.PIC16F181,
        Bootloader_Device_Lists.NOSUPPORT.PIC16F180,
        Bootloader_Device_Lists.NOSUPPORT.PIC16F183,
        Bootloader_Device_Lists.NOSUPPORT.PIC16F153,
        Bootloader_Device_Lists.NOSUPPORT.PIC16F184,
        Bootloader_Device_Lists.NOSUPPORT.PIC16F191,
    );
    switch (archString) {
        case "AVR":
            // build the not supported array and return
            return notSupported.concat(
                Bootloader_Device_Lists.SUPPORTED.PIC16F152,
                Bootloader_Device_Lists.SUPPORTED.PIC16F188,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ40,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ41,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ43,
            );
        case "PIC18":
            return notSupported.concat(
                Bootloader_Device_Lists.SUPPORTED.PIC16F152,
                Bootloader_Device_Lists.SUPPORTED.PIC16F188,
                Bootloader_Device_Lists.SUPPORTED.AVRDA,
                Bootloader_Device_Lists.SUPPORTED.Tiny0,
                Bootloader_Device_Lists.SUPPORTED.Tiny1,
            );
        case "PIC16":
            return notSupported.concat(
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ40,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ41,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ43,
                Bootloader_Device_Lists.SUPPORTED.AVRDA,
                Bootloader_Device_Lists.SUPPORTED.Tiny0,
                Bootloader_Device_Lists.SUPPORTED.Tiny1,
            );
        case "TINY0":
            return notSupported.concat(
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ40,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ41,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ43,
                Bootloader_Device_Lists.SUPPORTED.AVRDA,
                Bootloader_Device_Lists.SUPPORTED.Tiny1,
            );
        case "TINY1":
            return notSupported.concat(
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ40,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ41,
                Bootloader_Device_Lists.SUPPORTED.PIC18FQ43,
                Bootloader_Device_Lists.SUPPORTED.AVRDA,
                Bootloader_Device_Lists.SUPPORTED.Tiny0,
            );
        default:
            return undefined;
    }
};
/**
 * The data in this file is specific to the Melody Bootloader 8bit module
 */
export const Bootloader_Device_Lists = {
    SUPPORTED: {
        AVRDA: ["AVR128DA48", "AVR64DA64", "avr32da28", "avr32da32"],
        Tiny0: ["attiny1607", "attiny807", "attiny404", "attiny202"],
        Tiny1: ["attiny3217", "attiny1617", "attiny212", "attiny413", "ATtiny816"],
        PIC18FQ40: ["PIC18F04Q40", "PIC18F15Q40"],
        PIC18FQ41: ["PIC18F04Q41", "PIC18F15Q41"],
        PIC18FQ43: ["PIC18F57Q43", "PIC18F26Q43"],
        PIC16F152: ["PIC16F15276", "PIC16F15254", "PIC16F15244"],
        PIC16F188: ["PIC16F18854", "PIC16LF18875", "PIC16LF18855", "PIC16f18875"],
    },
    NOSUPPORT: {
        PIC18FQ10: ["PIC18F24Q10", "PIC18F47Q10"],
        PIC18FQ83: ["PIC18F57Q83", "PIC18F26Q83"],
        PIC18FQ84: ["PIC18F57Q84", "PIC18F26Q84"],
        PIC18FQ71: ["PIC18F26Q71", "PIC18F56Q71", "PIC18F24Q71", "PIC18F45Q71"],
        PIC18FQ20: ["PIC18F06Q20", "PIC18F16Q20"],
        PIC18FQ24: ["PIC18F26Q24", "PIC18F45Q24", "PIC18F56Q24"],
        PIC18FK40: ["PIC18F24K40", "PIC18F66K40", "PIC18LF45K40"],
        PIC18FK42: ["PIC18F26K42", "pic18lf45k42", "pic18lf56k42"],
        Tiny2: ["attiny1624", "ATtiny827", "ATTINY424", "ATTINY3227"],
        AVREA: ["AVR64EA48", "avr32EA28", "AVR16EA32"],
        AVRDB: ["AVR128DB48", "AVR64DB64", "AVR32DB28", "AVR128DB32"],
        AVRDU: ["AVR64DU28", "AVR64DU32"],
        AVRDD: ["AVR64DD28", "AVR32DD32", "AVR16DD14", "AVR32DD20"],
        Mega: ["Atmega328PB", "ATmega4809", "ATmega3208", "ATmega1609", "ATmega809"],
        PIC16F171: ["PIC16F17146", "PIC16F17126", "PIC16F17114", "PIC16LF17144"],
        PIC16F181: ["PIC16F18126", "PIC16F18146", "PIC16F18125", "PIC16LF18145"],
        PIC16F180: ["PIC16F18026", "PIC16F18046", "PIC16F18056", "PIC16LF18075"],
        PIC16F183: ["PIC16F18313", "PIC16F18323", "PIC16F18326", "PIC16LF18346"],
        PIC16F153: ["PIC16F15313", "pic16LF15345", "PIC16F15386"],
        PIC16F184: ["pic16F18424", "PIC16lf18456", "PIC16F18444"],
        PIC16F191: ["PIC16F19155", "PIC16lf19196", "pic16f19175"],
    },
};

const AVR_DATA = {
    architecure: "AVR" as Architecture,
    supportedDevices: Bootloader_Device_Lists.SUPPORTED.AVRDA.concat(
        Bootloader_Device_Lists.SUPPORTED.Tiny0,
        Bootloader_Device_Lists.SUPPORTED.Tiny1,
    ),
    notSupportedDevices: getNotSupportedForArchTests("AVR"),
};

const PIC18_DATA = {
    architecure: "PIC18" as Architecture,
    supportedDevices: Bootloader_Device_Lists.SUPPORTED.PIC18FQ43.concat(
        Bootloader_Device_Lists.SUPPORTED.PIC18FQ40,
        Bootloader_Device_Lists.SUPPORTED.PIC18FQ41,
    ),
    notSupportedDevices: getNotSupportedForArchTests("PIC18"),
};

const PIC16_DATA = {
    architecure: "PIC16" as Architecture,
    supportedDevices: Bootloader_Device_Lists.SUPPORTED.PIC16F152.concat(
        Bootloader_Device_Lists.SUPPORTED.PIC16F188,
    ),
    notSupportedDevices: getNotSupportedForArchTests("PIC16"),
};

const TINY0_DATA = {
    architecure: "AVR" as Architecture,
    supportedDevices: Bootloader_Device_Lists.SUPPORTED.Tiny0,
    notSupportedDevices: getNotSupportedForArchTests("TINY0"),
};

const TINY1_DATA = {
    architecure: "AVR" as Architecture,
    supportedDevices: Bootloader_Device_Lists.SUPPORTED.Tiny1,
    notSupportedDevices: getNotSupportedForArchTests("TINY1"),
};

const GoodMockPinData: any = { PC6: { cname: "TESTNAME", high: true } }; // correct data
const BadMockPinData: any = { PC6: { cname: "", high: "true" } }; // incorrect data types
const EmptyMockPinData: any = {}; // no data

// Exported API
export const DeviceTestData = {
    PIC18: PIC18_DATA,
    PIC16: PIC16_DATA,
    AVR: AVR_DATA,
    TINY1: TINY1_DATA,
    TINY0: TINY0_DATA,
};
export const PinsTestData = {
    Good: GoodMockPinData,
    Bad: BadMockPinData,
    Empty: EmptyMockPinData,
};

export const MockDelayHeader: HeaderFile = {
    name: "delay.h",
    path: "timer/",
};

export const testIndicatorData: pin_row = {
    name: "BL_Indicator",
    module: "BOOT",
    function: "INDICATE",
    direction: direction.output,
    filter: {
        module: "GPIO",
        aliasReEx: ".*",
    },
    behaviour: behaviour.OPTIONAL_PIN_MUX,
    attribs: [
        {
            cname: "BL_INDICATOR",
            analog: false,
            high: true,
            aliasReEx: ".*",
        },
    ],
};

export const testEntryData: pin_row = {
    name: "BL_Entry",
    module: "BOOT",
    function: "ENTRY",
    direction: direction.input,
    filter: {
        module: "GPIO",
        aliasReEx: ".*",
    },
    behaviour: behaviour.OPTIONAL_PIN_MUX,
    attribs: [
        {
            cname: "BL_ENTRY",
            analog: false,
            wpu: true,
            high: false,
            aliasReEx: ".*",
        },
    ],
};

export const testUSARTPayload = {
    moduleName: "USART1",
    Read: {
        name: "Read",
        returns: "uint8_t",
    },
    Write: {
        name: "Write",
        returns: "void",
    },
    isRxReady: {
        name: "IsRxReady",
        returns: "bool",
    },
    isTxReady: {
        name: "IsTxReady",
        returns: "bool",
    },
    TransmitDone: {
        name: "IsTxDone",
        returns: "bool",
    },
    AutoBaudSet: {
        name: "AutoBaudSet",
        returns: "void",
    },
    AutoBaudQuery: {
        name: "AutoBaudQuery",
        returns: "bool",
    },
    AutoBaudDetectError: "USART1_IsAutoBaudDetectError",
    AutoBaudReset: "USART1_AutoBaudDetectErrorReset",
};

export const testUARTPayload = {
    ...testUSARTPayload,
    moduleName: "UART1",
};

// ********************* Data for building template test data *************************************
/**
 * This Data structure represents a valid UART payload
 * that will be read in in the event of a UART connection from the derived data class
 */
export const attachedUARTPayload: BootTemplatePayload = {
    BL_UARTAutoBaudQuery: "UART1_AutoBaudQuery",
    BL_UARTAutoBaudSet: "UART1_AutoBaudSet",
    BL_UARTIsRxReady: "UART1_IsRxReady",
    BL_UARTIsTxReady: "UART1_IsTxReady",
    BL_UARTRead: "UART1_Read",
    BL_UARTTransmitDone: "UART1_IsTxDone",
    BL_UARTWrite: "UART1_Write",
    BL_USARTAutoBaudDetectErrorReset: "UART1_AutoBaudDetectOverflowReset",
    BL_USARTIsAutoBaudDetectError: "UART1_IsAutoBaudDetectOverflow",
};

export const attachedUSARTPayload: BootTemplatePayload = {
    BL_UARTWrite: "USART1_Write",
    BL_UARTRead: "USART1_Read",
    BL_UARTIsRxReady: "USART1_IsRxReady",
    BL_UARTIsTxReady: "USART1_IsTxReady",
    BL_UARTTransmitDone: "USART1_IsTxDone",
    BL_UARTAutoBaudSet: "USART1_AutoBaudSet",
    BL_UARTAutoBaudQuery: "USART1_AutoBaudQuery",
    BL_USARTIsAutoBaudDetectError: "USART1_IsAutoBaudDetectError",
    BL_USARTAutoBaudDetectErrorReset: "USART1_AutoBaudDetectErrorReset",
};

/**
 * This data structure represents an un-attached uart payload (Default import)
 */
export const undefinedUARTPayload: BootTemplatePayload = {
    BL_UARTRead: "unimplemented",
    BL_UARTIsRxReady: "unimplemented",
    BL_UARTWrite: "unimplemented",
    BL_UARTIsTxReady: "unimplemented",
    BL_UARTTransmitDone: "unimplemented",
    BL_UARTAutoBaudQuery: "unimplemented",
    BL_UARTAutoBaudSet: "unimplemented",
    BL_USARTAutoBaudDetectErrorReset: "undefined_AutoBaudDetectErrorReset",
    BL_USARTIsAutoBaudDetectError: "undefined_IsAutoBaudDetectError",
};

/**
 * This data structure is used as the starting data
 * structure to build the devices specific template data
 */
export const defaultTemplateData: BootTemplatePayload = {
    DELAY_HEADER_PATH: "NotApplicable",
    CHECKSUM_SIZE_U: "2",
    NVM_ACCESS_TYPE: "BYTE_ACCESS",
    ...undefinedUARTPayload,
    RESET_BIT_U: "NotApplicable",
    FUSE: "NotApplicable",
    AVR_PAGE: "NotApplicable",
    EEPROM_SIZE_U: "0x00",
    BOOT_BLOCK_SIZE_U: "512bytes",
    DEVICE_TYPE: "NotApplicable",
    FLASH_SIZE_U: "0x00",
    EEPROM_START_U: "0x00",
    DEVICE_ID_START_ADDRESS: "0x3FFFFE",
    CHIP_TYPE: undefined,
    DRAIN_RXBUF_AFTER_AB: "NotApplicable",
    DEVICE_FAMILY: "NotApplicable",
    PIC18F_Q: "Disabled",
    SUPPORT_CONFIG_WRITE: "Disabled",
    SUPPORT_CONFIG_READ: "Disabled",
    APP_VALID_VALUE_U: "0x55",
    NO_APPLICATION_CODE_U: "0xFF",
    CONFIGURATIONS_BYTES_START_ADDRESS_U: "0x300000",
    USER_ID_START_ADDRESS_U: "0x200000",
    EECON1_SFR: "NVMCON1",
    EECON2_SFR: "NVMLOCK",
    EEDATL_SFR: "NVMDATL",
    EEDATH_SFR: "NVMDATH",
    EEADRL_SFR: "NVMADRL",
    EEADRH_SFR: "NVMADRH",
    CFGS_BIT: "NVMREGS",
    BOOTLOADER_RUNNING_IO: "Disabled",
    IO_PIN_ENTRY: "Disabled",
    SUPPORT_FLASH_READ: "Disabled",
    SUPPORT_EE_DATA_READ: "Disabled",
    SUPPORT_EE_DATA_WRITE: "Disabled",
    SUPPORT_CHECKSUM: "Disabled",
    ERASE_CONTROL: "FullErase",
    VERIFY: "RESET_VECTOR_VERIFICATION",
    CRC_SEED_U: "NotApplicable",
    CRC_SIZE_U: "NotApplicable",
    CRC_POLYNOMIAL_U: "NotApplicable",
    CRC_XOR_U: "0x00000000",
    NEW_RESET_VECTOR_U: "0x00",
    NEW_INTERRUPT_VECTOR_LOW_U: "NotApplicable",
    NEW_INTERRUPT_VECTOR_HIGH_U: "NotApplicable",
    SELF_PROTECTION: "Disabled",
    BL_ENTRY_POINT: "BL_ENTRY",
    BL_INDICATOR_PIN: "BL_INDICATOR",
    BL_INDICATOR_PIN_ACTIVATION: "Active_High",
};

/**
 * Device Specific template data for AVR128DA48
 */
export const AVR128DA48_TestTemplateData: BootTemplatePayload = {
    ...defaultTemplateData,
    DELAY_HEADER_PATH: "timer/delay.h",
    ...attachedUSARTPayload,
    AVR_PAGE: "Expanded",
    APP_VALID_VALUE_U: "0x55",
    BOOT_BLOCK_SIZE_U: "512bytes",
    FLASH_SIZE_U: "0x20000",
    CFGS_BIT: "NVMREGS",
    BL_INDICATOR_PIN_ACTIVATION: "Active_Low",
    BOOTLOADER_RUNNING_IO: "Enabled",
    RESET_BIT_U: "RSTCTRL_SWRST_bm",
    BL_INDICATOR_PIN: "TESTNAME",
    EEPROM_START_U: "0x1400",
    EEPROM_SIZE_U: "0x200",
    CHIP_TYPE: "AVR",
    DEVICE_TYPE: "AVR",
    FUSE: "Reserved_Two_Byte",
};

/**
 * Device Specific template data for ATtiny1607
 */
export const ATTINY1607_TestTemplateData: BootTemplatePayload = {
    ...defaultTemplateData,
    DELAY_HEADER_PATH: "timer/delay.h",
    ...attachedUSARTPayload,
    AVR_PAGE: "Classic",
    APP_VALID_VALUE_U: "0x55",
    BOOT_BLOCK_SIZE_U: "256bytes",
    CFGS_BIT: "NVMREGS",
    BL_INDICATOR_PIN_ACTIVATION: "Active_High",
    EEPROM_START_U: "0x1400",
    FLASH_SIZE_U: "0xBFFF",
    EEPROM_SIZE_U: "0x100",
    IO_PIN_ENTRY: "Enabled",
    NVM_ACCESS_TYPE: "ROW_ACCESS",
    RESET_BIT_U: "RSTCTRL_SWRE_bm",
    BL_ENTRY_POINT: "TESTNAME",
    CHIP_TYPE: "TINY",
    DEVICE_TYPE: "AVR",
    FUSE: "Reserved_Two_Byte",
};

/**
 * Device Specific template data for ATtiny3217
 */
export const ATTINY3217_TestTemplateData: BootTemplatePayload = {
    ...defaultTemplateData,
    DELAY_HEADER_PATH: "timer/delay.h",
    ...attachedUSARTPayload,
    AVR_PAGE: "Classic",
    APP_VALID_VALUE_U: "0x55",
    BOOT_BLOCK_SIZE_U: "256bytes",
    CFGS_BIT: "NVMREGS",
    BL_INDICATOR_PIN_ACTIVATION: "Active_High",
    EEPROM_SIZE_U: "0x100",
    EEPROM_START_U: "0x1400",
    IO_PIN_ENTRY: "Enabled",
    NVM_ACCESS_TYPE: "ROW_ACCESS",
    FLASH_SIZE_U: "0xFFFF",
    RESET_BIT_U: "RSTCTRL_SWRE_bm",
    BL_ENTRY_POINT: "TESTNAME",
    CHIP_TYPE: "TINY",
    DEVICE_TYPE: "AVR",
    FUSE: "Reserved_One_Byte",
};

/**
 * Device Specific template data for PIC18F57Q43
 */
export const PIC18F57Q43_TestTemplateData: BootTemplatePayload = {
    ...defaultTemplateData,
    DELAY_HEADER_PATH: "timer/delay.h",
    ...attachedUARTPayload,
    NEW_RESET_VECTOR_U: "0x1388",
    NEW_INTERRUPT_VECTOR_HIGH_U: "0x1390",
    EEPROM_START_U: "0x380000",
    EEPROM_SIZE_U: "0x400",
    FLASH_SIZE_U: "0x20000",
    NEW_INTERRUPT_VECTOR_LOW_U: "0x13A0",
    SUPPORT_CONFIG_WRITE: "Disabled",
    SUPPORT_CONFIG_READ: "Enabled",
    PIC18F_Q: "Enabled",
    DEVICE_TYPE: "PIC",
    CHIP_TYPE: "PIC18",
    DRAIN_RXBUF_AFTER_AB: "Disabled",
    BOOTLOADER_RUNNING_IO: "Enabled",
    BL_INDICATOR_PIN: "TESTNAME",
    BL_INDICATOR_PIN_ACTIVATION: "Active_Low",
};
